with Ada.Finalization;
with Ada.Unchecked_Deallocation;
with Ada.Numerics.Discrete_Random;
with Ada.Containers.Doubly_Linked_Lists; -- External
with Game.Screen; -- Akanvader
package Implementation is

	type Base is new Ada.Finalization.Limited_Controlled with null record;
	type Base_Access is access all Base;
	procedure Dispose is new Ada.Unchecked_Deallocation(Base, Base_Access);

	type Point is 
		record
			X, Y: Integer;
		end record;

	type Direction is (Left, Right);

	type Actor is abstract new Base with 
		record
			Location: Point;
			Width, Height: Natural;
			Deleted: Boolean := False;
		end record;

	type Actor_Access is access all Actor'Class;
	package Actor_List is new Ada.Containers.Doubly_Linked_Lists(Actor_Access);

	procedure Advance(Object: access Actor; List: in out Actor_List.List) is abstract;
	procedure Draw(Object: in out Actor; Screen: in out Game.Screen.Screen_Buffer) is abstract;
	procedure Collide(Object: in out Actor; Other: Actor_Access) is abstract;

	procedure Test_Hit(List: in out Actor_List.List; It: access Actor'Class);

	type Fighter is new Actor with 
		record
			Shooting_Interval: Natural;
			Last_Moved: Direction;
		end record;

	procedure Initialize(Object: in out Fighter);
	procedure Advance(Object: access Fighter; List: in out Actor_List.List);
	procedure Draw(Object: in out Fighter; Screen: in out Game.Screen.Screen_Buffer);
	procedure Collide(Object: in out Fighter; Other: Actor_Access);
	procedure Move(Object: in out Fighter; Way: Direction);
	function Shooting_Location(Object: in Fighter) return Point;

	type Fighter_Access is access Fighter;

	subtype Enemy_Shooting_Probability is Natural range 0..199;
	package Random is new Ada.Numerics.Discrete_Random(Enemy_Shooting_Probability);
	Seed: Random.Generator;

	type Enemy is new Actor with
		record
			Way: Direction;
			Way_Step: Integer;
			Way_Length: Integer;
			Interval: Natural;
		end record;

	procedure Initialize(Object: in out Enemy);
	procedure Advance(Object: access Enemy; List: in out Actor_List.List);
	procedure Draw(Object: in out Enemy; Screen: in out Game.Screen.Screen_Buffer);
	procedure Collide(Object: in out Enemy; Other: Actor_Access);

	type Enemy_Access is access Enemy;

	type Shot is abstract new Actor with 
		record
			Interval: Natural;
		end record;

	procedure Initialize(Object: in out Shot);
	procedure Draw(Object: in out Shot; Screen: in out Game.Screen.Screen_Buffer);
	procedure Collide(Object: in out Shot; Other: Actor_Access);

	type Shot_Access is access Shot'Class;

	type Fighter_Shot is new Shot with null record;

	procedure Advance(Object: access Fighter_Shot; List: in out Actor_List.List);

	type Enemy_Shot is new Shot with null record;

	procedure Advance(Object: access Enemy_Shot; List: in out Actor_List.List);

	type Shield is new Actor with
		record
			Life: Natural;
		end record;

	procedure Initialize(Object: in out Shield);
	procedure Advance(Object: access Shield; List: in out Actor_List.List);
	procedure Draw(Object: in out Shield; Screen: in out Game.Screen.Screen_Buffer);
	procedure Collide(Object: in out Shield; Other: Actor_Access);

	type Shield_Access is access Shield;

end Implementation;
